%Returns the roots to the radii polynomial for the approximate connecting
%orbit pair (x0,a).
function [R1,R2]=Connecting_Orbit_Validation(x0,a)
format long
[~, L]=size(x0);

%Homoclinic function
function [f]=f_intval(x_int,aa_int,d)
        f=intval(zeros(L,1));
        P_x_intval=polyval(P1u_x,d,x_int(L-1));
        P_y_intval=polyval(P1u_y,d,x_int(L-1));
        Q_x_intval=polyval(P1s_x,d,x_int(L));
        Q_y_intval=polyval(P1s_y,d,x_int(L));        
        f(1)=P_x_intval-x_int(1);
        f(2)=P_y_intval-x_int(2);

        for i=3:L-2
            if mod(i,2)==0
                f(i)=b_int*x_int(i-3)-x_int(i);
            elseif mod(i,2)==1
                f(i)=intval(1)-aa_int*x_int(i-2)^2+x_int(i-1)-x_int(i);
             end
        end
        f(L-1)=intval(1)-aa_int*x_int(L-3)^2+x_int(L-2)-Q_x_intval;
        f(L)=b_int*x_int(L-3)-Q_y_intval;
end


    function [Df]=Df_intval(x_int,aa_int,d)
        Df=intval(zeros(L,L));
        dP_x_intval=polyval(P1ux_theta,d,x_int(L-1));
        dP_y_intval=polyval(P1uy_theta,d,x_int(L-1));
        dQ_x_intval=polyval(P1sx_theta,d,x_int(L));
        dQ_y_intval=polyval(P1sy_theta,d,x_int(L));
        for i=1:L
            for j=1:L
                if i==1
                    if j==1
                    Df(i,j)=-intval(1);
                    elseif j==L-1;
                            Df(i,j)=dP_x_intval;
                    end
                elseif i==2
                    if j==2
                        Df(i,j)=-intval(1);
                    elseif j==L-1
                            Df(i,j)=dP_y_intval;
                    end
                elseif i==L-1
                    if j==L-3
                        Df(i,j)=-intval(2)*aa_int*x_int(j);
                    elseif j==L-2
                        Df(i,j)=intval(1);
                    elseif j==L
                        Df(i,j)=-dQ_x_intval;
                    end
                elseif i==L
                        if j==L-3
                        Df(i,j)=b_int;
                    elseif j==L
                        Df(i,j)=-dQ_y_intval;
                        end
                elseif i>2 && i<=L-2 && mod(i,2)==1
                    if j==i-2
                        Df(i,j)=-intval(2)*aa_int*x_int(j);
                    elseif j==i-1
                        Df(i,j)=intval(1);
                    elseif j==i
                        Df(i,j)=-intval(1);
                    end
                 elseif i>2 && i<=L-2 && mod(i,2)==0
                     if j==i
                         Df(i,j)=-intval(1);
                     elseif j==i-3
                         Df(i,j)=b_int;
                     end
                end
            end
                     
 
        end
    end
a_int=intval(a);
b_int = intval('0.3');
x1_int=intval(x0);
d=0;
%Parameterization Controls:
M2 = 15;
N2 = 25;

scale_eigs = intval(1);

%Parameter Domains:
tauBar1 = intval('0.1325');
tau = intval('0.1325');   %0.23 works
nu = intval('0.95');
%First Fixed Point:



[P1s_MNx, P1s_MNy, delta1s] = manifoldBranch_1Stable(...
    a_int,  b_int, M2, N2, tauBar1,tau, nu, scale_eigs);

[P1s_x, P1s_y] = convertToPoly_2Dbranch_2Dspace(P1s_MNx, P1s_MNy, M2, N2);

[P1u_MNx, P1u_MNy, delta1u] = manifoldBranch_1Unstable(...
    a_int,  b_int, M2, N2, tauBar1,tau, nu, scale_eigs);

[P1u_x, P1u_y] = convertToPoly_2Dbranch_2Dspace(P1u_MNx, P1u_MNy, M2, N2);




P1ux_theta = pderiv(P1u_x,'x', 1);  %First Derivative of P1 wrt theta
P1uy_theta=pderiv(P1u_y,'x', 1);
P1sx_theta = pderiv(P1s_x,'x', 1);
P1sy_theta=pderiv(P1s_y,'x', 1);

P1ux_theta2 = pderiv(P1u_x,'x', 2);  %Second Derivative of P1 wrt theta
P1uy_theta2=pderiv(P1u_y,'x', 2);
P1sx_theta2 = pderiv(P1s_x,'x', 2);
P1sy_theta2=pderiv(P1s_y,'x', 2);


P1ux_tau = pderiv(P1u_x,'t', 1);  %First Derivative of P1 wrt tau
P1uy_tau=pderiv(P1u_y,'t', 1);
P1sx_tau = pderiv(P1s_x,'t', 1);
P1sy_tau=pderiv(P1s_y,'t', 1);

P1ux_tau2 = pderiv(P1u_x,'t', 2);  %Second Derivative of P1 wrt tau
P1uy_tau2=pderiv(P1u_y,'t', 2);
P1sx_tau2 = pderiv(P1s_x,'t', 2);
P1sy_tau2=pderiv(P1s_y,'t', 2);

P1ux_tautheta=pderiv(P1ux_theta,'t',1);  %Mixed Derivative of P1
P1uy_tautheta=pderiv(P1uy_theta,'t',1);
P1sx_tautheta=pderiv(P1sx_theta,'t',1);
P1sy_tautheta=pderiv(P1sy_theta,'t',1);

P1ux_theta3 = pderiv(P1u_x,'x', 3);  %Third Derivative of P1 wrt theta
P1uy_theta3=pderiv(P1u_y,'x', 3);
P1sx_theta3 = pderiv(P1s_x,'x', 3);
P1sy_theta3=pderiv(P1s_y,'x', 3);

P1ux_tautheta2=pderiv(P1ux_theta2,'t',1);  %Mixed Derivative of P1
P1uy_tautheta2=pderiv(P1uy_theta2,'t',1);
P1sx_tautheta2=pderiv(P1sx_theta2,'t',1);
P1sy_tautheta2=pderiv(P1sy_theta2,'t',1);

P1ux_tau2theta=pderiv(P1ux_tau2,'x',1);  %Mixed Derivative of P1
P1uy_tau2theta=pderiv(P1uy_tau2,'x',1);
P1sx_tau2theta=pderiv(P1sx_tau2,'x',1);
P1sy_tau2theta=pderiv(P1sy_tau2,'x',1);

P1ux_tau3 = pderiv(P1u_x,'t', 3);  %Third Derivative of P1 wrt theta
P1uy_tau3=pderiv(P1u_y,'t', 3);
P1sx_tau3 = pderiv(P1s_x,'t', 3);
P1sy_tau3=pderiv(P1s_y,'t', 3);


%Second Fixed Point:
% 
% [P2s_MNx, P2s_MNy, delta2s] = manifoldBranch_2Stable(...
%     a2,  b2, M2, N2, tauBar1,tau, nu, scale_eigs);
% 
% 
% [P2u_MNx, P2u_MNy, delta2u] = manifoldBranch_2Untable(...
%     a2,  b2, M2, N2, tauBar1,tau, nu, scale_eigs);


%-------------------------Y-Bound------------------------------------------
f_error=intval(zeros(L,1));
f_error(1)=delta1u;
f_error(2)=delta1u;
f_error(L-1)=delta1s;
f_error(L)=delta1s;
A=inv(Df_intval(x1_int,a_int,d));
Y_bd=abs(Df_intval(x1_int,a_int,d)\(f_intval(x1_int,a_int,d)+f_error));

%-------------------------Z-Bound------------------------------------------
delta=infsup(-1,1);
x1_star=x1_int+delta*10^(-4);
    function [D2f]=D2f_int(x_int)
D2f=intval(zeros(L,L,L));
for k=1:L-2
    if mod(k,2)==1
        D2f(k+2,k,k)=-2*a_int;
    end
end   
D2f(1,L-1,L-1)=polyval(P1ux_theta2,d,x_int(L-1));
D2f(2,L-1,L-1)=polyval(P1uy_theta2,d,x_int(L-1));
D2f(L-1,L,L)=polyval(P1sx_theta2,d,x_int(L));
D2f(L,L,L)=polyval(P1sy_theta2,d,x_int(L));
    end
Z=D2f_int(x1_star);
ZQ_sum=intval(zeros(L,L));
for i3=1:L
    ZQ_sum=ZQ_sum+mag(Z(:,:,i3));
end

%-------------Linear terms in radii polynomials---------------------------
DP_x=intval(2*pi*delta1u/(nu*-log((x1_int(5)+nu)/(2*nu))));
DQ_x=intval(2*pi*delta1s/(nu*-log((x1_int(6)+nu)/(2*nu))));
ZL_error=intval(zeros(L,L));
ZL_error(1,L-1)=DP_x;
ZL_error(2,L-1)=DP_x;
ZL_error(L-1,L)=DQ_x;
ZL_error(L,L)=DQ_x;
ZL=abs(intval(eye(L,L)-A*(Df_intval(x1_int,a_int,d)+ZL_error)))*ones(L,1);



%------Quadratic terms in radii polynomials---------------------------
ZQ_error=intval(zeros(L,L));
ZQ_error(1,L-1)=DP_x^2;
ZQ_error(2,L-1)=DP_x^2;
ZQ_error(L-1,L)=DQ_x^2;
ZQ_error(L,L)=DQ_x^2;

ZQ=abs(A)*((ZQ_sum+ZQ_error)*ones(L,1));

%--------Calculating roots-----------------------------------------
r_1=ZL-intval(ones(L,1));
r_2=ZQ;
R2=(-r_1(:)+sqrt(r_1(:).^2-4*r_2(:).*Y_bd(:)))./(2.*r_2(:));
R1=(-r_1(:)-sqrt(r_1(:).^2-4*r_2(:).*Y_bd(:)))./(2.*r_2(:));

end
