%Takes in two approximate orbits x1 and x2 at the parameter values a1, a2
%respectively and validates a family of connecting orbits from (x1,a1) to
%(x2,a2).  Returning the roots RR1, RR2 to the radii polynomials.
function [DWD,RR1,RR2,x1_int]=Continuation_Validation(x1,x2,a1,a2)
format long
[~, L]=size(x1);
%a=1.4: .5 .3 -.5 1.5 -1 4
%a=1.4;
%a1=1.22447;
% a1=1.3855;
% b=0.3;


a1_int=intval(a1*1000)/intval(1000);
a2_int=intval(a2*1000)/intval(1000);
a_int=intval(1245)/intval(1000);%a1_int;
b_int = intval('0.3');



%Parameterization Controls: 
M2 = 15;
N2 = 25;

scale_eigs = intval(1);

%Parameter Domains:
tauBar1 = intval('0.1325');
tau = intval('0.1325');   %0.23 works
nu = intval('0.95');
%First Fixed Point:



[P1s_MNx, P1s_MNy, delta1s] = manifoldBranch_1Stable(...
    a_int,  b_int, M2, N2, tauBar1,tau, nu, scale_eigs);

[P1s_x, P1s_y] = convertToPoly_2Dbranch_2Dspace(P1s_MNx, P1s_MNy, M2, N2);

[P1u_MNx, P1u_MNy, delta1u] = manifoldBranch_1Unstable(...
    a_int,  b_int, M2, N2, tauBar1,tau, nu, scale_eigs);

[P1u_x, P1u_y] = convertToPoly_2Dbranch_2Dspace(P1u_MNx, P1u_MNy, M2, N2);




P1ux_theta = pderiv(P1u_x,'x', 1);  %First Derivative of P1 wrt theta
P1uy_theta=pderiv(P1u_y,'x', 1);
P1sx_theta = pderiv(P1s_x,'x', 1);
P1sy_theta=pderiv(P1s_y,'x', 1);

P1ux_theta2 = pderiv(P1u_x,'x', 2);  %Second Derivative of P1 wrt theta
P1uy_theta2=pderiv(P1u_y,'x', 2);
P1sx_theta2 = pderiv(P1s_x,'x', 2);
P1sy_theta2=pderiv(P1s_y,'x', 2);


P1ux_tau = pderiv(P1u_x,'t', 1);  %First Derivative of P1 wrt tau
P1uy_tau=pderiv(P1u_y,'t', 1);
P1sx_tau = pderiv(P1s_x,'t', 1);
P1sy_tau=pderiv(P1s_y,'t', 1);

P1ux_tau2 = pderiv(P1u_x,'t', 2);  %Second Derivative of P1 wrt tau
P1uy_tau2=pderiv(P1u_y,'t', 2);
P1sx_tau2 = pderiv(P1s_x,'t', 2);
P1sy_tau2=pderiv(P1s_y,'t', 2);

P1ux_tautheta=pderiv(P1ux_theta,'t',1);  %Mixed Derivative of P1
P1uy_tautheta=pderiv(P1uy_theta,'t',1);
P1sx_tautheta=pderiv(P1sx_theta,'t',1);
P1sy_tautheta=pderiv(P1sy_theta,'t',1);

P1ux_theta3 = pderiv(P1u_x,'x', 3);  %Third Derivative of P1 wrt theta
P1uy_theta3=pderiv(P1u_y,'x', 3);
P1sx_theta3 = pderiv(P1s_x,'x', 3);
P1sy_theta3=pderiv(P1s_y,'x', 3);

P1ux_tautheta2=pderiv(P1ux_theta2,'t',1);  %Mixed Derivative of P1
P1uy_tautheta2=pderiv(P1uy_theta2,'t',1);
P1sx_tautheta2=pderiv(P1sx_theta2,'t',1);
P1sy_tautheta2=pderiv(P1sy_theta2,'t',1);

P1ux_tau2theta=pderiv(P1ux_tau2,'x',1);  %Mixed Derivative of P1
P1uy_tau2theta=pderiv(P1uy_tau2,'x',1);
P1sx_tau2theta=pderiv(P1sx_tau2,'x',1);
P1sy_tau2theta=pderiv(P1sy_tau2,'x',1);

P1ux_tau3 = pderiv(P1u_x,'t', 3);  %Third Derivative of P1 wrt theta
P1uy_tau3=pderiv(P1u_y,'t', 3);
P1sx_tau3 = pderiv(P1s_x,'t', 3);
P1sy_tau3=pderiv(P1s_y,'t', 3);
%Second Fixed Point:
% 
% [P2s_MNx, P2s_MNy, delta2s] = manifoldBranch_2Stable(...
%     a2,  b2, M2, N2, tauBar1,tau, nu, scale_eigs);
% 
% 
% [P2u_MNx, P2u_MNy, delta2u] = manifoldBranch_2Untable(...
%     a2,  b2, M2, N2, tauBar1,tau, nu, scale_eigs);


%Homoclinic function

function [g_intval]=gn_intval(x_int,aa_int,d)
        g_intval=intval(zeros(L,1));
        P_x_intval=polyval(P1u_x,d,x_int(L-1));
        P_y_intval=polyval(P1u_y,d,x_int(L-1));
        Q_x_intval=polyval(P1s_x,d,x_int(L));
        Q_y_intval=polyval(P1s_y,d,x_int(L));        
        g_intval(1)=P_x_intval-x_int(1);
        g_intval(2)=P_y_intval-x_int(2);

        for i=3:L-2
            if mod(i,2)==0
                g_intval(i)=b_int*x_int(i-3)-x_int(i);
            elseif mod(i,2)==1
                g_intval(i)=intval(1)-aa_int*x_int(i-2)^2+x_int(i-1)-x_int(i);
             end
        end
        g_intval(L-1)=intval(1)-aa_int*x_int(L-3)^2+x_int(L-2)-Q_x_intval;
        g_intval(L)=b_int*x_int(L-3)-Q_y_intval;
end

    function [J2_intval]=jacob2_intval(x_int,aa_int,d)
        J2_intval=intval(zeros(L,L));
        dP_x_intval=polyval(P1ux_theta,d,x_int(L-1));
        dP_y_intval=polyval(P1uy_theta,d,x_int(L-1));
        dQ_x_intval=polyval(P1sx_theta,d,x_int(L));
        dQ_y_intval=polyval(P1sy_theta,d,x_int(L));
        for i=1:L
            for j=1:L
                if i==1
                    if j==1
                    J2_intval(i,j)=-intval(1);
                    elseif j==L-1
                            J2_intval(i,j)=dP_x_intval;
                    end
                elseif i==2
                    if j==2
                        J2_intval(i,j)=-intval(1);
                    elseif j==L-1
                            J2_intval(i,j)=dP_y_intval;
                    end
                elseif i==L-1
                    if j==L-3
                        J2_intval(i,j)=-intval(2)*aa_int*x_int(j);
                    elseif j==L-2
                        J2_intval(i,j)=intval(1);
                    elseif j==L
                        J2_intval(i,j)=-dQ_x_intval;
                    end
                elseif i==L
                        if j==L-3
                        J2_intval(i,j)=b_int;
                    elseif j==L
                        J2_intval(i,j)=-dQ_y_intval;
                        end
                elseif i>2 && i<=L-2 && mod(i,2)==1
                    if j==i-2
                        J2_intval(i,j)=-intval(2)*aa_int*x_int(j);
                    elseif j==i-1
                        J2_intval(i,j)=intval(1);
                    elseif j==i
                        J2_intval(i,j)=-intval(1);
                    end
                 elseif i>2 && i<=L-2 && mod(i,2)==0
                     if j==i
                         J2_intval(i,j)=-intval(1);
                     elseif j==i-3
                         J2_intval(i,j)=b_int;
                     end
                end
            end
                     
 
        end
    end

function x=Newton_Henon(x0,a)
N = 2000; % define max. number of iterations 
epsilon = 1e-13; % define tolerance 
maxval = 20000.0; % define value for divergence 
xx = mid(x0); % load initial guess     
       
      
while (N>0) 
 %JJ = feval(jacobnxn,xx);
 %JJ=mid(jacob2_intval(intval(xx),aa,aa-a_int));
 JJ=mid(jacob2_intval(xx,a,a-a_int));
  xn = xx - (JJ\mid(gn_intval(xx,a,a-a_int)))';
% xr(:,N)=mid(gn_intval(xn,a,a-a_int));
if abs(det(JJ))<epsilon 
 error('newtonm - Jacobian is singular - try new x0'); 
 abort; 
end
% xn = xx - inv(JJ)*feval(fn,xx);
 %xn = xx - (inv(JJ)*mid(gn_intval(intval(xx),aa,aa-a_int)))';

%if abs(feval(fn,xn))<epsilon 
 %if norm(mid(gn_intval(intval(xn),aa,aa-a_int)))<epsilon 
 if norm(mid(gn_intval(xn,a,a-a_int)))<epsilon
 x=xn; 
 iter = 2000-N; 
 disp(['iterations = ',num2str(iter)]); 
 return; 
 end
%if abs(feval(fn,xx))>maxval 
 if norm(mid(gn_intval(xn,a,a-a_int)))>maxval 
 iter = 2000-N; 
 disp(['iterations = ',num2str(iter)]); 
 error('Solution diverges'); 
 abort;
 end
 N = N - 1; 
 xx = xn; 
end

error('No convergence after 2000 iterations.'); 
abort; 
end 
xx1=Newton_Henon(x1,a1);
xx2=Newton_Henon(x2,a2);

x1_int=intval(xx1);
x2_int=intval(xx2);


%A=inv(jacob2_intval(x_int))*gn_intval(x_int)'
counter=0;
x_guess=intval(zeros((1200000-1140000)/1+2,6));
%x_guess(1,:)=intval([.5 .3 -.5 1.5 -1 4]);
%x_guess(1,:)=intval([1.250839945180841 0.027542218393661 -1.131606566081153 0.375251983554252 -0.959726811285197 2.616326180751014]);
%x_guess(1,:)=intval([1.26186610812973   0.00643835779339  -1.14317484324934   0.37855983243891  -1.06335714625913   2.55884767820850]);%<--a=1.35
%x_guess(1,:)=intval([1.25339331996716  -0.00023124434204  -1.13678419211416   0.37601799599015  -1.09264226305985   2.51829275021104]);%<---a=1.38
R1=intval(zeros(1,L));
R2=intval(zeros(1,L));
r_2=intval(zeros((1200000-1140000)/1,L));
r_1=intval(zeros((1200000-1140000)/1,L));
Ybd=intval(zeros((1200000-1140000)/1,L));
%y_0=intval(zeros(1,L)); 
%y_1=intval(zeros(1,L));
y_2=intval(zeros(1,L));
y_3=intval(zeros(1,L));
Ey_0=intval(zeros(1,L));
Ey_1=intval(zeros(1,L));
Ey_2=intval(zeros(1,L));
Ey_3=intval(zeros(1,L));
g1=zeros((1200000-1140000)/1,L);
%  for i=1.4:-0.05:1.35
    %  counter=counter+1;
%  %continuation paramaters
%  % a1=1.4;
%  % a2=1.398;
%  a1=i;
%  a2=i-0.05;


%------------Validiation of continuation bounds: Y,Z(r)--------------------

%----Error Bounds for manifolds---------------
x5_max=max(abs(x1_int(L-1)),abs(x2_int(L-1)));
x6_max=max(abs(x1_int(L)),abs(x2_int(L)));
DH1_x=intval(2*pi*delta1u/(nu*-log((x5_max+nu)/(2*nu))));
DG1_x=intval(2*pi*delta1s/(nu*-log((x6_max+nu)/(2*nu))));
a_max=max(abs(a1_int),abs(a2_int));%<--------- continuation parameter
DH1_a=2*pi*delta1u/(tau*-log((a_max+tau)/(2*tau)));
DG1_a=2*pi*delta1s/(tau*-log((a_max+tau)/(2*tau)));
%---------------------------Y-bound----------------------------------------

delta=infsup(0,1);
b=infsup(-10^-2,10^-2);
a_star=a1_int+delta*(a2_int-a1_int);
s=(1-delta)*x1_int(L-1)+delta*x2_int(L-1);
t=(1-delta)*x1_int(L)+delta*x2_int(L);

y_0=gn_intval(x1_int,a1_int,a2_int-a1_int)';%<--g(0)
 
y_1a=intval(zeros(1,L));
y_la(1)=polyval(P1ux_tau,0,x1_int(L-1));
y_1a(2)=polyval(P1uy_tau,0,x1_int(L-1));
y_1a(L-1)=-x1_int(L-3)^2-polyval(P1sx_tau,0,x1_int(L));
y_1a(L)=-polyval(P1sy_tau,0,x1_int(L));
for i=3:L-2
    if mod(i,2)==1
        y_1a(i)=-x1_int(i-2)^2;
    end
end

%y_1(counter,:)=(x2_int-x1_int)*jacob2_intval(x1_int,a1_int,a1_int-a_int)+[polyval(P1ux_tau,a1_int-a_int,x1_int(5)),polyval(P1uy_tau,a1_int-a_int,x1_int(5)),-x1_int(1)^2,0,-x1_int(3)^2-polyval(P1sx_tau,a1_int-a_int,x1_int(6)),-polyval(P1sy_tau,a1_int-a_int,x1_int(6))]*(a2_int-a1_int);%<---g'(0)
y_1=(x2_int-x1_int)*jacob2_intval(x1_int,a1_int,0)+y_1a*(a2_int-a1_int);%<---g'(0)


y_2(1)=polyval(P1ux_theta2,0,x1_int(L-1))*(x2_int(L-1)-x1_int(L-1))^2+2*polyval(P1ux_tautheta,0,x1_int(L-1))*(x2_int(L-1)-x1_int(L-1))*(a2_int-a1_int)+polyval(P1ux_tau2,0,x1_int(L-1))*(a2_int-a1_int)^2;
y_2(2)=polyval(P1uy_theta2,0,x1_int(L-1))*(x2_int(L-1)-x1_int(L-1))^2+2*polyval(P1uy_tautheta,0,x1_int(L-1))*(a2_int-a1_int)*(x2_int(L-1)-x1_int(L-1))+polyval(P1uy_tau2,0,x1_int(L-1))*(a2_int-a1_int)^2;
y_2(L-1)=-2*(x2_int(L-3)-x1_int(L-3))*(a1_int*(x2_int(L-3)-x1_int(L-3))+x1_int(L-3)*(a2_int-a1_int))-2*(a2_int-a1_int)*(x2_int(L-3)-x1_int(L-3))*x1_int(L-3)-polyval(P1sx_theta2,0,x1_int(L))*(x2_int(L)-x1_int(L))^2-2*polyval(P1sx_tautheta,0,x1_int(L))*(a2_int-a1_int)*(x2_int(L)-x1_int(L))-polyval(P1sx_tau2,0,x1_int(L))*(a2_int-a1_int)^2;
y_2(L)=-polyval(P1sy_theta2,0,x1_int(L))*(x2_int(L)-x1_int(L))^2-2*polyval(P1sy_tautheta,0,x1_int(L))*(a2_int-a1_int)*(x2_int(L)-x1_int(L))-polyval(P1sy_tau2,0,x1_int(L))*(a2_int-a1_int)^2;
for i=3:L-2
    if mod(i,2)==1
        y_2(i)=-2*(x2_int(L-2)-x1_int(L-2))*(a1_int*(x2_int(L-2)-x1_int(L-2))+x1_int(L-2)*(a2_int-a1_int))-2*(a2_int-a1_int)*(x2_int(L-2)-x1_int(L-2))*x1_int(L-2);
    end
end

% y_2(counter,:)=[polyval(P1ux_theta2,a1_int-a_int,x1_int(5))*(x2_int(5)-x1_int(5))^2+2*polyval(P1ux_tautheta,a1_int-a_int,x1_int(5))*(x2_int(5)-x1_int(5))*(a2_int-a1_int)+polyval(P1ux_tau2,a1_int-a_int,x1_int(5))*(a2_int-a1_int)^2,polyval(P1uy_theta2,a1_int-a_int,x1_int(5))*(x2_int(5)-x1_int(5))^2+2*polyval(P1uy_tautheta,a1_int-a_int,x1_int(5))*(a2_int-a1_int)*(x2_int(5)-x1_int(5))+polyval(P1uy_tau2,a1_int-a_int,x1_int(5))*(a2_int-a1_int)^2 ...
%        -2*(x2_int(1)-x1_int(1))*(a1_int*(x2_int(1)-x1_int(1))+x1_int(1)*(a2_int-a1_int))-2*(a2_int-a1_int)*(x2_int(1)-x1_int(1))*x1_int(1),0,-2*(x2_int(3)-x1_int(3))*(a1_int*(x2_int(3)-x1_int(3))+x1_int(3)*(a2_int-a1_int))-2*(a2_int-a1_int)*(x2_int(3)-x1_int(3))*x1_int(3)-polyval(P1sx_theta2,a1_int-a_int,x1_int(6))*(x2_int(6)-x1_int(6))^2-2*polyval(P1sx_tautheta,a1_int-a_int,x1_int(6))*(a2_int-a1_int)*(x2_int(6)-x1_int(6))-polyval(P1sx_tau2,a1_int-a_int,x1_int(6))*(a2_int-a1_int)^2 ...
%        -polyval(P1sy_theta2,a1_int-a_int,x1_int(6))*(x2_int(6)-x1_int(6))^2-2*polyval(P1sy_tautheta,a1_int-a_int,x1_int(6))*(a2_int-a1_int)*(x2_int(6)-x1_int(6))-polyval(P1sy_tau2,a1_int-a_int,x1_int(6))*(a2_int-a1_int)^2]; %<--------g''(0)
  

y_3(1)=polyval(P1ux_theta3,a_star-a1_int,s)*(x2_int(L-1)-x1_int(L-1))^3+3*polyval(P1ux_tautheta2,a_star-a1_int,s)*(a2_int-a1_int)*(x2_int(L-1)-x1_int(L-1))^2+3*polyval(P1ux_tau2theta,a_star-a1_int,s)*(a2_int-a1_int)^2*(x2_int(L-1)-x1_int(L-1))+polyval(P1ux_tau3,a_star-a1_int,s)*(a2_int-a1_int)^3;
y_3(2)=polyval(P1uy_theta3,a_star-a1_int,s)*(x2_int(L-1)-x1_int(L-1))^3+3*polyval(P1uy_tautheta2,a_star-a1_int,s)*(a2_int-a1_int)*(x2_int(L-1)-x1_int(L-1))^2+3*polyval(P1uy_tau2theta,a_star-a1_int,s)*(a2_int-a1_int)^2*(x2_int(L-1)-x1_int(L-1))+polyval(P1uy_tau3,a_star-a1_int,s)*(a2_int-a1_int)^3; 
y_3(L-1)=-4*(x2_int(L-3)-x1_int(L-3))^2*(a2_int-a1_int)-2*x1_int(L-3)*(a2_int-a1_int)*(x2_int(L-3)-x1_int(L-3))-polyval(P1sx_theta3,a_star-a1_int,t)*(x2_int(L)-x1_int(L))^3-3*polyval(P1sx_tautheta2,a_star-a1_int,t)*(a2_int-a1_int)*(x2_int(L)-x1_int(L))^2-3*polyval(P1sx_tau2theta,a_star-a1_int,t)*(a2_int-a1_int)^2*(x2_int(L)-x1_int(L))-polyval(P1sx_tau3,a_star-a1_int,t)*(a2_int-a1_int)^3;
y_3(L)=-polyval(P1sy_theta3,a_star-a1_int,t)*(x2_int(L)-x1_int(L))^3-3*polyval(P1sy_tautheta2,a_star-a1_int,t)*(a2_int-a1_int)*(x2_int(L)-x1_int(L))^2-3*polyval(P1sy_tau2theta,a_star-a1_int,t)*(a2_int-a1_int)^2*(x2_int(L)-x1_int(L))-polyval(P1sy_tau3,a_star-a1_int,t)*(a2_int-a1_int)^3;
for i=3:L-2
    if mod(i,2)==1
        y_3(i)=-4*(x2_int(L-2)-x1_int(L-2))^2*(a2_int-a1_int)-2*x1_int(L-2)*(a2_int-a1_int)*(x2_int(L-2)-x1_int(L-2));
    end
end
% y_3(counter,:)=[polyval(P1ux_theta3,a_star-a_int,s)*(x2_int(5)-x1_int(5))^3+3*polyval(P1ux_tautheta2,a_star-a_int,s)*(a2_int-a1_int)*(x2_int(5)-x1_int(5))^2+3*polyval(P1ux_tau2theta,a_star-a_int,s)*(a2_int-a1_int)^2*(x2_int(5)-x1_int(5))+polyval(P1ux_tau3,a_star-a_int,s)*(a2_int-a1_int)^3 ...
%      polyval(P1uy_theta3,a_star-a_int,s)*(x2_int(5)-x1_int(5))^3+3*polyval(P1uy_tautheta2,a_star-a_int,s)*(a2_int-a1_int)*(x2_int(5)-x1_int(5))^2+3*polyval(P1uy_tau2theta,a_star-a_int,s)*(a2_int-a1_int)^2*(x2_int(5)-x1_int(5))+polyval(P1uy_tau3,a_star-a_int,s)*(a2_int-a1_int)^3 ...
%      -4*(x2_int(1)-x1_int(1))^2*(a2_int-a1_int)-2*x1_int(1)*(a2_int-a1_int)*(x2_int(1)-x1_int(1)),0,-4*(x2_int(3)-x1_int(3))^2*(a2_int-a1_int)-2*x1_int(3)*(a2_int-a1_int)*(x2_int(3)-x1_int(3))-polyval(P1sx_theta3,a_star-a_int,t)*(x2_int(6)-x1_int(6))^3-3*polyval(P1sx_tautheta2,a_star-a_int,t)*(a2_int-a1_int)*(x2_int(6)-x1_int(6))^2-3*polyval(P1sx_tau2theta,a_star-a_int,t)*(a2_int-a1_int)^2*(x2_int(6)-x1_int(6))-polyval(P1sx_tau3,a_star-a_int,t)*(a2_int-a1_int)^3 ...
%      -polyval(P1sy_theta3,a_star-a_int,t)*(x2_int(6)-x1_int(6))^3-3*polyval(P1sy_tautheta2,a_star-a_int,t)*(a2_int-a1_int)*(x2_int(6)-x1_int(6))^2-3*polyval(P1sy_tau2theta,a_star-a_int,t)*(a2_int-a1_int)^2*(x2_int(6)-x1_int(6))-polyval(P1sy_tau3,a_star-a_int,t)*(a2_int-a1_int)^3];%... %<---- g'''(d*)
 

Ey_0(1)=delta1u; %<--Error bound for g(0)
Ey_0(2)=delta1u;
Ey_0(L-1)=delta1s;
Ey_0(L)=delta1s;  

Ey_1(1)=abs(DH1_x*(x2_int(L-1)-x1_int(L-1)))+abs(DH1_a*(a2_int-a1_int));
Ey_1(2)=abs(DH1_x*(x2_int(L-1)-x1_int(L-1)))+abs(DH1_a*(a2_int-a1_int));
Ey_1(L-1)=abs(DG1_x*(x2_int(L)-x1_int(L)))+abs(DG1_a*(a2_int-a1_int));
Ey_1(L)=abs(DG1_x*(x2_int(L)-x1_int(L)))+abs(DG1_a*(a2_int-a1_int));
%Ey_1(counter,:)=abs([DH1_x*(x2_int(5)-x1_int(5)),DH1_x*(x2_int(5)-x1_int(5)),0,0,DG1_x*(x2_int(6)-x1_int(6)),DG1_x*(x2_int(6)-x1_int(6))])+abs([DH1_a*(a2_int-a1_int),DH1_a*(a2_int-a1_int),0,0,DG1_a*(a2_int-a1_int),DG1_a*(a2_int-a1_int)]);  %<---Error bound for g'(0)
  
Ey_2(1)=abs((DH1_x)^2*(x2_int(L-1)-x1_int(L-1))^2/delta1u+2*DH1_x*DH1_a*(a2_int-a1_int)*(x2_int(L-1)-x1_int(L-1))/delta1u+DH1_a^2*(a2_int-a1_int)^2/delta1u);
Ey_2(2)=abs((DH1_x)^2*(x2_int(L-1)-x1_int(L-1))^2/delta1u+2*DH1_x*DH1_a*(a2_int-a1_int)*(x2_int(L-1)-x1_int(L-1))/delta1u+DH1_a^2*(a2_int-a1_int)^2/delta1u);
Ey_2(L-1)=abs(DG1_x^2*(x2_int(L)-x1_int(L))^2/delta1s+2*DG1_x*DG1_a*(a2_int-a1_int)*(x2_int(L)-x1_int(L))/delta1s+DG1_a^2*(a2_int-a1_int)^2/delta1s);
Ey_2(L)=abs(DG1_x^2*(x2_int(L)-x1_int(L))^2/delta1s+2*DG1_x*DG1_a*(a2_int-a1_int)*(x2_int(L)-x1_int(L))/delta1s+DG1_a^2*(a2_int-a1_int)^2/delta1s);

%Ey_2(counter,:)=abs([(DH1_x)^2*(x2_int(5)-x1_int(5))^2/delta1u+2*DH1_x*DH1_a*(a2_int-a1_int)*(x2_int(5)-x1_int(5))/delta1u+DH1_a^2*(a2_int-a1_int)^2/delta1u ...   %<------error mbound for g''(0)
%         (DH1_x)^2*(x2_int(5)-x1_int(5))^2/delta1u+2*DH1_x*DH1_a*(a2_int-a1_int)*(x2_int(5)-x1_int(5))/delta1u+DH1_a^2*(a2_int-a1_int)^2/delta1u,0,0 ...
%         DG1_x^2*(x2_int(6)-x1_int(6))^2/delta1s+2*DG1_x*DG1_a*(a2_int-a1_int)*(x2_int(6)-x1_int(6))/delta1s+DG1_a^2*(a2_int-a1_int)^2/delta1s ...
%         DG1_x^2*(x2_int(6)-x1_int(6))^2/delta1s+2*DG1_x*DG1_a*(a2_int-a1_int)*(x2_int(6)-x1_int(6))/delta1s+DG1_a^2*(a2_int-a1_int)^2/delta1s]);

Ey_3(1)=abs(DH1_x^3*(x2_int(L-1)-x1_int(L-1))^3/delta1u^2+DH1_a*DH1_x^2*(x2_int(L-1)-x1_int(L-1))^2*((a2_int-a1_int)+(x2_int(L-1)-x1_int(L-1)))/delta1u^2+DH1_a^2*DH1_x*(a2_int-a1_int)*(x2_int(L-1)-x1_int(L-1))^2/delta1u^2);
Ey_3(2)=abs(DH1_x^3*(x2_int(L-1)-x1_int(L-1))^3/delta1u^2+DH1_a*DH1_x^2*(x2_int(L-1)-x1_int(L-1))^2*((a2_int-a1_int)+(x2_int(L-1)-x1_int(L-1)))/delta1u^2+DH1_a^2*DH1_x*(a2_int-a1_int)*(x2_int(L-1)-x1_int(L-1))^2/delta1u^2);
Ey_3(L-1)=abs(DG1_x^3*(x2_int(L)-x1_int(L))^3/delta1s^2+DG1_a*DG1_x^2*(x2_int(L)-x1_int(L))^2*((a2_int-a1_int)+(x2_int(L)-x1_int(L)))/delta1s^2+DG1_a^2*DG1_x*(a2_int-a1_int)*(x2_int(L)-x1_int(L))^2/delta1s^2);
Ey_3(L)=abs(DG1_x^3*(x2_int(L)-x1_int(L))^3/delta1s^2+DG1_a*DG1_x^2*(x2_int(L)-x1_int(L))^2*((a2_int-a1_int)+(x2_int(L)-x1_int(L)))/delta1s^2+DG1_a^2*DG1_x*(a2_int-a1_int)*(x2_int(L)-x1_int(L))^2/delta1s^2);

% Ey_3(counter,:)=abs([DH1_x^3*(x2_int(5)-x1_int(5))^3/delta1u^2+DH1_a*DH1_x^2*(x2_int(5)-x1_int(5))^2*((a2_int-a1_int)+(x2_int(5)-x1_int(5)))/delta1u^2+DH1_a^2*DH1_x*(a2_int-a1_int)*(x2_int(5)-x1_int(5))^2/delta1u^2 ... <----Error bound for g'''(d*)
%         DH1_x^3*(x2_int(5)-x1_int(5))^3/delta1u^2+DH1_a*DH1_x^2*(x2_int(5)-x1_int(5))^2*((a2_int-a1_int)+(x2_int(5)-x1_int(5)))/delta1u^2+DH1_a^2*DH1_x*(a2_int-a1_int)*(x2_int(5)-x1_int(5))^2/delta1u^2,0,0 ...
%         DG1_x^3*(x2_int(6)-x1_int(6))^3/delta1s^2+DG1_a*DG1_x^2*(x2_int(6)-x1_int(6))^2*((a2_int-a1_int)+(x2_int(6)-x1_int(6)))/delta1s^2+DG1_a^2*DG1_x*(a2_int-a1_int)*(x2_int(6)-x1_int(6))^2/delta1s^2 ... 
%         DG1_x^3*(x2_int(6)-x1_int(6))^3/delta1s^2+DG1_a*DG1_x^2*(x2_int(6)-x1_int(6))^2*((a2_int-a1_int)+(x2_int(6)-x1_int(6)))/delta1s^2+DG1_a^2*DG1_x*(a2_int-a1_int)*(x2_int(6)-x1_int(6))^2/delta1s^2]);

A_int=inv(jacob2_intval(x1_int,a1_int,a1_int-a_int));
Ybd=abs(A_int*y_0')+abs(A_int*y_1')+.5*abs(A_int*y_2')+(1/6)*mag(abs(A_int*y_3'))+abs(A_int)*(Ey_0+Ey_1+.5*Ey_2+(1/6)*Ey_3)';
%----------------------Computing Z-bounds----------------------------------
% x1_star=x1_int(1)-10^-4+delta*(x2_int(1)-x1_int(1)+2*10^-4);
% x3_star=x1_int(3)-10^-4+delta*(x2_int(3)-x1_int(3)+2*10^-4);
x_star=intval(zeros(L));
for i=1:L-2
%     x_star(i)=x1_int(i)-10^-4+delta*(x2_int(i)-x1_int(i)+2*10^-4);
      x_star(i)=x1_int(i)+delta*(x2_int(i)-x1_int(i))+b;
end
s_star=x1_int(L-1)+delta*(x2_int(L-1)-x1_int(L-1))+b;
t_star=x1_int(L)+delta*(x2_int(L)-x1_int(L))+b;

D_xx=intval(zeros(L,L));
D_xa=intval(zeros(L,L));
D_xxx=intval(zeros(L,L));
D_xxa=intval(zeros(L,L));
D_xaa=intval(zeros(L,L));

Error_D_xx=intval(zeros(L,L));
Error_D_xa=intval(zeros(L,L));
Error_D_xaa=intval(zeros(L,L));
Error_D_xxa=intval(zeros(L,L));
Error_D_xxx=intval(zeros(L,L));

D_xx(1,L-1)=polyval(P1ux_theta2,0,x1_int(L-1));
Error_D_xx(1,L-1)=DH1_x^2/delta1u;
D_xx(2,L-1)=polyval(P1uy_theta2,0,x1_int(L-1));
Error_D_xx(2,L-1)=DH1_x^2/delta1u;
D_xx(L-1,L)=-polyval(P1sx_theta2,0,x1_int(L));
Error_D_xx(L-1,L)=DG1_x^2/delta1s;
D_xx(L,L)=-polyval(P1sy_theta2,0,x1_int(L));
Error_D_xx(L,L)=DG1_x^2/delta1s;
for k=1:L-2
    if mod(k,2)==1
        D_xx(k+2,k)=-2*a1_int;
    end
end

D_xa(1,L-1)=polyval(P1ux_tautheta,0,x1_int(L-1));
Error_D_xa(1,L-1)=DH1_x*DH1_a/delta1u;
D_xa(2,L-1)=polyval(P1uy_tautheta,0,x1_int(L-1));
Error_D_xa(2,L-1)=DH1_x*DH1_a/delta1u;
D_xa(L-1,L)=-polyval(P1sx_tautheta,0,x1_int(L));
Error_D_xa(L-1,L)=DG1_x*DG1_a/delta1s;
D_xa(L,L)=-polyval(P1sy_tautheta,0,x1_int(L));
Error_D_xa(L,L)=DG1_x*DG1_a/delta1s;
for i=3:L
    if mod(i,2)==1
        D_xa(i,i-2)=-2*x1_int(i-2);
    end
end

D_xaa(1,L-1)=polyval(P1ux_tau2theta,a_star-a1_int,s_star)*(a2_int-a1_int)^2;
Error_D_xaa(1,L-1)=DH1_x*DH1_a^2/delta1u^2*(a2_int-a1_int)^2;
D_xaa(2,L-1)=polyval(P1uy_tau2theta,a_star-a1_int,s_star)*(a2_int-a1_int)^2;
Error_D_xaa(2,L-1)=DH1_x*DH1_a^2/delta1u^2*(a2_int-a1_int)^2;
D_xaa(L-1,L)=-polyval(P1sx_tau2theta,a_star-a1_int,t_star)*(a2_int-a1_int)^2;
Error_D_xaa(L-1,L)=DG1_x*DG1_a^2/delta1s^2*(a2_int-a1_int)^2;
D_xaa(L,L)=-polyval(P1sy_tau2theta,a_star-a1_int,t_star)*(a2_int-a1_int)^2;
Error_D_xaa(L,L)=DG1_x*DG1_a^2/delta1s^2*(a2_int-a1_int)^2;

D_xxa(1,L-1)=polyval(P1ux_tautheta2,a_star-a1_int,s_star)*(a2_int-a1_int);
Error_D_xxa(1,L-1)=DH1_x^2*DH1_a/delta1u^2*(a2_int-a1_int);
D_xxa(2,L-1)=polyval(P1uy_tautheta2,a_star-a1_int,s_star)*(a2_int-a1_int);
Error_D_xxa(2,L-1)=DH1_x^2*DH1_a/delta1u^2*(a2_int-a1_int);
D_xxa(L-1,L)=-polyval(P1sx_tautheta2,a_star-a1_int,t_star)*(a2_int-a1_int);
Error_D_xxa(L-1,L)=DG1_x^2*DG1_a/delta1s^2*(a2_int-a1_int);
D_xxa(L,L)=-polyval(P1sy_tautheta2,a_star-a1_int,t_star)*(a2_int-a1_int);
Error_D_xxa(L,L)=DG1_x^2*DG1_a/delta1s^2*(a2_int-a1_int);

for i=3:L
    if mod(i,2)==1
        D_xxa(i,i-2)=-2*x_star(i-2)*(a2_int-a1_int);
    end
end

D_xxx(1,L-1)=polyval(P1ux_theta3,a_star-a1_int,s_star)*(s_star-x1_int(L-1))^2;
Error_D_xxx(1,L-1)=DH1_x^3/delta1u^2*(s_star-x1_int(L-1))^2;
D_xxx(2,L-1)=polyval(P1uy_theta3,a_star-a1_int,s_star)*(s_star-x1_int(L-1))^2;
Error_D_xxx(2,L-1)=DH1_x^3/delta1u^2*(s_star-x1_int(L-1))^2;
D_xxx(L-1,L)=-polyval(P1sx_theta3,a_star-a1_int,t_star)*(t_star-x1_int(L))^2;
Error_D_xxx(L-1,L-1)=DG1_x^3/delta1s^2*(t_star-x1_int(L))^2;
D_xxx(L,L)=-polyval(P1sy_theta3,a_star-a1_int,t_star)*(t_star-x1_int(L))^2;
Error_D_xxx(L,L)=DG1_x^3/delta1s^2*(t_star-x1_int(L))^2;
%---Linear coefficient of radii poly--------------
x2_x1=intval(zeros(L,L));
x2_x1(1,L-1)=x2_int(L-1)-x1_int(L-1);
x2_x1(2,L-1)=x2_int(L-1)-x1_int(L-1);
x2_x1(L-1,L)=x2_int(L)-x1_int(L);
x2_x1(L,L)=x2_int(L)-x1_int(L);
for i=3:L
    if mod(i,2)==1
        x2_x1(i,i-2)=x2_int(i-2)-x1_int(i-2);
    end
end
%ZL=intval(zeros(L,L));
ZL=(abs(A_int*((D_xx+Error_D_xx).*x2_x1))+abs(A_int*(D_xa+Error_D_xa)*(a2_int-a1_int))+.5*abs(A_int)*(mag(D_xxx)+mag(Error_D_xxx))+abs(A_int)*(mag(D_xxa)+Error_D_xxa).*abs(x2_x1)+.5*abs(A_int)*(mag(D_xaa)+Error_D_xaa))*ones(L,1)-ones(L,1);

%ZQ=intval(zeros(L,L));
ZQ=(abs(A_int*(D_xx+Error_D_xx))+abs(A_int)*(mag(D_xxa)+Error_D_xxa))*ones(L,1);
DWD=ZL.^2-4*ZQ.*Ybd;










 
 
 %-------Quadratic term of radii poly------------------- 
%r_2=(abs(A*DJ3_intval)+abs(A*Error_DJx2_intval))*ones(6,1);

%r_2(counter,:)=(mag(abs(A_int)*abs(DJ3_intval))+abs(A_int)*abs(DJ2_intval)+abs(A_int)*abs(Error_DJ3_intval))*ones(6,1);

RR1=(-ZL+sqrt(ZL.^2-4*ZQ.*Ybd))./(2.*ZQ);
RR2=(-ZL-sqrt(ZL.^2-4*ZQ.*Ybd))./(2.*ZQ);
% for iii=1:6
% if R2(counter,iii)<0 || max(R2(counter,:))>min(R1(counter,:))
%     disp(['no radii polynomial root found for a=',a1_int]);
%     error('no radii polynomial root found for a='); 
%  abort;
%end
%endsize(R1)
end




