%sys_sol returns the solution to the Newton method applied to the saddle
%node function F_saddle with x_0 an approximate saddle node.
function [x1]=sys_sol(x_0)
format long
[~, L]=size(x_0);
b=0.3;

%Henon Parameters:
%a_int=intval(13855)/intval(10000);
a_int=intval(1300)/intval(1000);
%a_int = intval(13)/intval(10);
b_int = intval('0.3');


%Parameterization Controls:
M2 = 15;
N2 = 25;

scale_eigs = intval(1); 

%Parameter Domains:
tauBar1 = intval('0.1325');
tau = intval('0.1325');   %0.23 works
nu = intval('0.95');
%First Fixed Point:



[P1s_MNx, P1s_MNy, delta1s] = manifoldBranch_1Stable(...
    a_int,  b_int, M2, N2, tauBar1,tau, nu, scale_eigs);

[P1s_x, P1s_y] = convertToPoly_2Dbranch_2Dspace(P1s_MNx, P1s_MNy, M2, N2);

[P1u_MNx, P1u_MNy, delta1u] = manifoldBranch_1Unstable(...
    a_int,  b_int, M2, N2, tauBar1,tau, nu, scale_eigs);

[P1u_x, P1u_y] = convertToPoly_2Dbranch_2Dspace(P1u_MNx, P1u_MNy, M2, N2);




P1ux_theta = pderiv(P1u_x,'x', 1);  %First Derivative of P1 wrt theta
P1uy_theta=pderiv(P1u_y,'x', 1);
P1sx_theta = pderiv(P1s_x,'x', 1);
P1sy_theta=pderiv(P1s_y,'x', 1);

P1ux_theta2 = pderiv(P1u_x,'x', 2);  %Second Derivative of P1 wrt theta
P1uy_theta2=pderiv(P1u_y,'x', 2);
P1sx_theta2 = pderiv(P1s_x,'x', 2);
P1sy_theta2=pderiv(P1s_y,'x', 2);


P1ux_tau = pderiv(P1u_x,'t', 1);  %First Derivative of P1 wrt tau
P1uy_tau=pderiv(P1u_y,'t', 1);
P1sx_tau = pderiv(P1s_x,'t', 1);
P1sy_tau=pderiv(P1s_y,'t', 1);

P1ux_tau2 = pderiv(P1u_x,'t', 2);  %Second Derivative of P1 wrt tau
P1uy_tau2=pderiv(P1u_y,'t', 2);
P1sx_tau2 = pderiv(P1s_x,'t', 2);
P1sy_tau2=pderiv(P1s_y,'t', 2);

P1ux_tautheta=pderiv(P1ux_theta,'t',1);  %Mixed Derivative of P1
P1uy_tautheta=pderiv(P1uy_theta,'t',1);
P1sx_tautheta=pderiv(P1sx_theta,'t',1);
P1sy_tautheta=pderiv(P1sy_theta,'t',1);

P1ux_theta3 = pderiv(P1u_x,'x', 3);  %Third Derivative of P1 wrt theta
P1uy_theta3=pderiv(P1u_y,'x', 3);
P1sx_theta3 = pderiv(P1s_x,'x', 3);
P1sy_theta3=pderiv(P1s_y,'x', 3);

P1ux_tautheta2=pderiv(P1ux_theta2,'t',1);  %Mixed Derivative of P1
P1uy_tautheta2=pderiv(P1uy_theta2,'t',1);
P1sx_tautheta2=pderiv(P1sx_theta2,'t',1);
P1sy_tautheta2=pderiv(P1sy_theta2,'t',1);

P1ux_tau2theta=pderiv(P1ux_tau2,'x',1);  %Mixed Derivative of P1
P1uy_tau2theta=pderiv(P1uy_tau2,'x',1);
P1sx_tau2theta=pderiv(P1sx_tau2,'x',1);
P1sy_tau2theta=pderiv(P1sy_tau2,'x',1);

P1ux_tau3 = pderiv(P1u_x,'t', 3);  %Third Derivative of P1 wrt theta
P1uy_tau3=pderiv(P1u_y,'t', 3);
P1sx_tau3 = pderiv(P1s_x,'t', 3);
P1sy_tau3=pderiv(P1s_y,'t', 3);
%Second Fixed Point:
% 
% [P2s_MNx, P2s_MNy, delta2s] = manifoldBranch_2Stable(...
%     a2,  b2, M2, N2, tauBar1,tau, nu, scale_eigs);
% 
% 
% [P2u_MNx, P2u_MNy, delta2u] = manifoldBranch_2Untable(...
%     a2,  b2, M2, N2, tauBar1,tau, nu, scale_eigs);


%Homoclinic function


function [g_intval]=gn_intval(x_int,aa_int,d)
        g_intval=intval(zeros(L,1));
        P_x_intval=polyval(P1u_x,d,x_int(L-1));
        P_y_intval=polyval(P1u_y,d,x_int(L-1));
        Q_x_intval=polyval(P1s_x,d,x_int(L));
        Q_y_intval=polyval(P1s_y,d,x_int(L));        
        g_intval(1)=P_x_intval-x_int(1);
        g_intval(2)=P_y_intval-x_int(2);

        for i=3:L-2
            if mod(i,2)==0
                g_intval(i)=b_int*x_int(i-3)-x_int(i);
            elseif mod(i,2)==1
                g_intval(i)=intval(1)-aa_int*x_int(i-2)^2+x_int(i-1)-x_int(i);
             end
        end
        g_intval(L-1)=intval(1)-aa_int*x_int(L-3)^2+x_int(L-2)-Q_x_intval;
        g_intval(L)=b_int*x_int(L-3)-Q_y_intval;
end



    function [J2_intval]=jacob2_intval(x_int,aa_int,d)
        J2_intval=intval(zeros(L,L));
        dP_x_intval=polyval(P1ux_theta,d,x_int(L-1));
        dP_y_intval=polyval(P1uy_theta,d,x_int(L-1));
        dQ_x_intval=polyval(P1sx_theta,d,x_int(L));
        dQ_y_intval=polyval(P1sy_theta,d,x_int(L));
        for i=1:L
            for j=1:L
                if i==1
                    if j==1
                    J2_intval(i,j)=-intval(1);
                    elseif j==L-1
                            J2_intval(i,j)=dP_x_intval;
                    end
                elseif i==2
                    if j==2
                        J2_intval(i,j)=-intval(1);
                    elseif j==L-1
                            J2_intval(i,j)=dP_y_intval;
                    end
                elseif i==L-1
                    if j==L-3
                        J2_intval(i,j)=-intval(2)*aa_int*x_int(j);
                    elseif j==L-2
                        J2_intval(i,j)=intval(1);
                    elseif j==L
                        J2_intval(i,j)=-dQ_x_intval;
                    end
                elseif i==L
                        if j==L-3
                        J2_intval(i,j)=b_int;
                    elseif j==L
                        J2_intval(i,j)=-dQ_y_intval;
                        end
                elseif i>2 && i<=L-2 && mod(i,2)==1
                    if j==i-2
                        J2_intval(i,j)=-intval(2)*aa_int*x_int(j);
                    elseif j==i-1
                        J2_intval(i,j)=intval(1);
                    elseif j==i
                        J2_intval(i,j)=-intval(1);
                    end
                 elseif i>2 && i<=L-2 && mod(i,2)==0
                     if j==i
                         J2_intval(i,j)=-intval(1);
                     elseif j==i-3
                         J2_intval(i,j)=b_int;
                     end
                end
            end
                     
 
        end
    end

    function [F]=F_saddle(x)
        x_int=intval(x(1:L));
        a3_int=intval(x(L+1));
        vv=x(L+2:2*L+1)';
        F=zeros(1,2*L+1);
        F(1:L)=mid(gn_intval(x_int,a3_int,a3_int-a_int));
        F(L+1)=norm(vv)^2-1;
        F(L+2:2*L+1)=mid(jacob2_intval(x_int,a3_int,a3_int-a_int))*vv;
        F=F';
    end

    function [DF]=DF_saddle(x)
        x_int=intval(x(1:L));
        aa_int=intval(x(L+1));
        JJ=jacob2_intval(x_int,aa_int,aa_int-a_int);
        V=x(L+2:2*L+1);
        DF=zeros(2*L+1,2*L+1);
        DF(1:L,1:L)=mid(JJ);
        DF(L+2:2*L+1,L+2:2*L+1)=mid(JJ);
        DF(L+1,L+2:2*L+1)=2*V;
        DF(1,L+1)=mid(polyval(P1ux_tau,aa_int-a_int,x_int(L-1)));
        DF(2,L+1)=mid(polyval(P1uy_tau,aa_int-a_int,x_int(L-1)));
        for i=3:L-2
            if mod(i,2)==1
                DF(i,L+1)=-mid(x_int(i-2))^2;
            end
        end
        DF(L-1,L+1)=-mid(x_int(L-3))^2-mid(polyval(P1sx_tau,aa_int-a_int,x_int(L)));
        DF(L,L+1)=-mid(polyval(P1sy_tau,aa_int-a_int,x_int(L)));
        DF(L+2,L-1)=mid(polyval(P1ux_theta2,aa_int-a_int,x_int(L-1)))*V(L-1);
        DF(L+2,L+1)=mid(polyval(P1ux_tautheta,aa_int-a_int,x_int(L-1)))*V(L-1);
        DF(L+3,L-1)=mid(polyval(P1uy_theta2,aa_int-a_int,x_int(L-1)))*V(L-1);
        DF(L+3,L+1)=mid(polyval(P1uy_tautheta,aa_int-a_int,x_int(L-1)))*V(L-1);
        for i=3:L-2
            if mod(i,2)==1
                DF(L+1+i,i-2)=-2*mid(aa_int)*V(i-2);
                DF(L+1+i,L+1)=-2*mid(x_int(i-2))*V(i-2);
            end
            
        end
        DF(2*L,L-3)=-2*mid(aa_int)*V(L-3);
        DF(2*L,L)=-mid(polyval(P1sx_theta2,aa_int-a_int,x_int(L)))*V(L);
        DF(2*L,L+1)=-2*mid(aa_int)*V(L-3)-mid(polyval(P1sx_tautheta,aa_int-a_int,x_int(L)))*V(L);
        DF(2*L+1,L)=-mid(polyval(P1sy_theta2,aa_int-a_int,x_int(L)))*V(L);
        DF(2*L+1,L+1)=-mid(polyval(P1sy_tautheta,aa_int-a_int,x_int(L)))*V(L);
    end


h=ones(1,37)*0.00001;
E=F_saddle(x_0+h)-F_saddle(x_0)-DF_saddle(x_0)*h';        
FF=F_saddle(x_0);
e=norm(E);
A1=norm(inv(DF_saddle(x_0)));
A2=norm(DF_saddle(x_0)\F_saddle(x_0));
function x = Newton_Henon(x0)%x = Newton_Henon(x0,aa,v)
N = 2000; % define max. number of iterations 
epsilon = 1e-13; % define tolerance 
maxval = 20000.0; % define value for divergence 
xx = mid(x0); % load initial guess 
            
             
while (N>0) 
 %JJ = feval(jacobnxn,xx);
 %JJ=mid(jacob2_intval(intval(xx),aa,aa-a_int));
 JJ=DF_saddle(xx);

% xn = xx - inv(JJ)*feval(fn,xx);
 %xn = xx - (inv(JJ)*mid(gn_intval(intval(xx),aa,aa-a_int)))';
 xn = xx - (JJ\F_saddle(xx))';
%if abs(feval(fn,xn))<epsilon 
 %if norm(mid(gn_intval(intval(xn),aa,aa-a_int)))<epsilon
 if norm(F_saddle(xn))<epsilon 
 x=xn; 
 iter = 2000-N;
 disp(['iterations = ',num2str(iter)]); 
 return; 
 end

%if abs(feval(fn,xx))>maxval 
 if norm(F_saddle(xx))>maxval 
 iter = 2000-N; 
 disp(['iterations = ',num2str(iter)]); 
 error('Solution diverges'); 
 abort;
 end 
 N = N - 1; 
 xx = xn; 
end 
error('No convergence after 2000 iterations.'); 
abort; 
end 

x1=Newton_Henon(x_0);
%counter=0;
%x_guess=intval(zeros((1200000-1140000)/1+2,6));
%x_guess(1,:)=intval([.5 .3 -.5 1.5 -1 4]);
%x_guess(1,:)=intval([1.250839945180841 0.027542218393661 -1.131606566081153 0.375251983554252 -0.959726811285197 2.616326180751014]);
%x_guess(1,:)=intval([1.26186610812973   0.00643835779339  -1.14317484324934   0.37855983243891  -1.06335714625913   2.55884767820850]);%<--a=1.35
%x_guess(1,:)=intval([1.25339331996716  -0.00023124434204  -1.13678419211416   0.37601799599015  -1.09264226305985   2.51829275021104]);%<---a=1.38


end














